// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/object_model.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/query.dart';

abstract class CrudRepository<Model extends ObjectModel> extends BaseRepository {
  FutureResult<void> create(Model object, {String? id});
  FutureResult<Model?> get(String id);
  FutureResult<List<Model?>> getAll();
  FutureResult<void> update(
    String id, {
    Model? object,
    Map<String, dynamic>? raw,
  });
  FutureResult<void> updateAll(Map<String, Object?> raw);
  FutureResult<void> delete(String id);
  FutureResult<void> deleteAll();
  FutureResult<List<Model?>> query(List<QueryInterface> conditions);
  StreamResult<List<Model?>> stream({
    String? id,
    List<QueryInterface>? conditions,
  });
}
